'' =================================================================================================
''
''   File....... jm_ds3231_lcd_demo.spin
''   Purpose....
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2021-2022 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon.mcphalen@gmail.com
''   Started....
''   Updated.... 26 JAN 2022
''
''   {$P1}
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                         ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us


con { terminal }

  BR_TERM  = 115_200                                            ' terminal baud rate

  #0, T_PST, T_ANSI                                             ' terminal types

  T_TYPE = T_PST


con { fixed io pins }

  PGM_RX   = 31  { I }                                          ' serial / programming
  PGM_TX   = 30  { O }

  EE_SDA   = 29  { I/O }                                        ' i2c / eeprom
  EE_SCL   = 28  { I/O }


con { app io pins }


con

  #true,  ON, OFF
  #false, NO, YES

  LCD_ADDR  = %111                                              ' LCD I2C address

  LCD_WIDTH =   16                                              ' LCD size (16 or 20)
  LCD_LINES =    2                                              ' 2 or 4


obj

' main                                                          ' * master Spin cog
  time : "jm_time_80"                                           '   timing and delays (80MHz)
  rtc  : "jm_ds3231"                                            '   DS3231 interface (I2C)
  lcd  : "jm_lcd_pcf8574"                                       '   PCF8574-based LCD (I2C)
  term : "jm_fullduplexserial"                                  ' * serial IO for terminal
  ansi : "jm_ansi"                                              '   ANSI terminal control sequences

' * uses cog when loaded


var

  byte  sc, mn, hr                                              ' time registers
  byte  dy, dt, mo, yr                                          ' date registers


pub main | last, tmp

  setup

' set_rtc

  last := -1                                                    ' force initial update

  repeat
    rtc.rd_time(@sc)                                            ' read current time
    if (sc <> last)                                             ' change?
      last := sc                                                ' mark this second
      rtc.rd_calendar(@dy)                                      ' read calendar

      lcd.crsr_home
      lcd.str(rtc.day_short(dy))                                ' day as string

      lcd.line(1)
      lcd.str(rtc.time_hhmmss(@sc))                             ' current time

      tmp := rtc.temp_c                                         ' temp in 0.01c
      lcd.crsr_xy(9, 0)
      lcd.jdpdec((tmp+5)/10, 1, 5, " ")                         ' print in 0.1c
      lcd.str(string($DF, "C"))

      tmp := tmp * 9_00 / 5_00 + 32_00                          ' convert to F
      lcd.crsr_xy(9, 1)
      lcd.jdpdec((tmp+5)/10, 1, 5, " ")                         ' print in 0.1f
      lcd.str(string($DF, "F"))

  repeat
    waitcnt(0)


pub set_rtc

'' Use to set initial time/date into RTC module

  rtc.set_time($00, $01, $20)
  rtc.set_calendar($04, $26, $01, $22)


pub setup

'' Setup IO and objects for application

  time.start                                                    ' setup timing & delays

  rtc.start(EE_SCL, EE_SDA)                                     ' connect to RTC

  lcd.start(LCD_ADDR, EE_SCL, EE_SDA)                           ' connect to LCD

  term.tstart(BR_TERM)                                          ' start serial for terminal *


pub wait_for_terminal(clear)

'' Wait for terminal to be open and key pressed

  term.rxflush
  term.rx
  if (clear)
    clear_screen


pub clear_screen

  if (T_TYPE == T_PST)
    term.tx(term#CLS)
  else
    term.str(ansi.hide_cursor)
    term.str(ansi.home)
    term.str(ansi.cls)


con { license }

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}